import numpy as np


def throw(ndarts):
    """Function that throws n darts

    This functions generates n sets of coordinates that are each drawn from a uniform distribution between -1 and 1

    Parameters
    ----------
    ndarts : int
        Number of coordinates to draw (darts to throw).

    Returns
    -------
    darts : array
        array of size (2xn) with the coordinates of darts thrown.
    """
    return (np.random.rand(2, ndarts) - 0.5) * 2


def is_in_unit_circle(dart):
    """Checks if a dart is in the unit circle

    function that checks whether a coordinate `dart` is inside the unit circle

    Parameters
    ----------
    dart : array
        Array of coordinates (dart positions) with size (2xn)

    Returns
    -------
    isincircle : array (boolean)
        Array of size n set to `True` where `dart` is in the unit circle, false otherwise.
    """

    # Distance to origin for each dart
    distance = np.sqrt(np.sum(dart**2, axis=0))

    isincircle = distance < 1

    return isincircle


def estimate_pi(ndarts):
    """Function that estimates pi using the 'throw of darts' method

    Parameters
    ----------
    ndarts : int
        number of darts to throw

    Returns
    -------
    pi : float
        An estimate for pi
    """
    darts = throw(ndarts)
    incircle = is_in_unit_circle(darts)
    return 4.0 * np.sum(incircle) / ndarts


def make_realisation(nrea, ndarts):
    """function that makes `nrea` realisations of `ndarts` dart throws and records the result

    Parameters
    ----------
    nrea : int
        number of realisation
    ndarts : int
        number of darts thrown at each realisation

    Returns
    -------
    pies : array
        array of the estimates of pi at each iteration
    """
    pies = []

    [pies.append(estimate_pi(ndarts)) for i in range(nrea)]
    return np.array(pies)


def get_pi_accuracy(nrea, nthrows):
    """
    Function that plots the mean and standard deviation of the results of `nrea` realisations
    of dart `nthrows` dart throws

    Parameters
    ----------
    nrea : int
        number of realisations of throws for each number of throws
    nthrows : array
        array containing numbers of throws for which we want `nrea` realisations

    Returns
    -------
    mean : array
        mean estimates
    std : array
        errors
    None
    """
    mean = []
    std = []
    for throw in nthrows:
        realisations = make_realisation(nrea, throw)
        mean.append(realisations.mean())
        std.append(realisations.std())
    return np.array(mean), np.array(std)



import matplotlib.pyplot as plt
import numpy as np

from .estimate_pi import *

def plot_darts(ndarts, circle=True):
    """
    Function that displays a throw of darts and highlights darts in the unit circle with a different colour

    Parameters
    ----------
    ndarts : int
        number of dart throws
    circle : bool, optional
        if True, draw the boundary

    Returns
    -------
    None
    """
    darts = throw(ndarts)
    incircle = is_in_unit_circle(darts)

    plt.figure(figsize=(5,5))
    plt.plot(darts[0, ~incircle], darts[1, ~incircle], '.', label = 'not in unit circle')
    plt.plot(darts[0,incircle], darts[1,incircle], '.', label = 'in unit circle')
    plt.gca().set_aspect('equal')
    plt.legend()

    if circle: draw_circle(r=1)
    return

def draw_circle(r=1,x0=0,y0=0):
    """ Simple function to draw a circle with radius r centered at x0, y0 on the current axis

    Parameters
    ----------
    r : float
        radius of the circle
    x0, y0 : float
        center of the circle
    """
    theta = np.linspace(0,2*np.pi)
    x = r*np.cos(theta)
    y = r*np.sin(theta)
    plt.plot(x,y,'k:')
